#!/bin/sh

 #######################################################################
 # Copyright 2007-2009 Igor Daniloff.
 # 
 # Following source code is the property of Doctor Web, Ltd.
 # Use is subject to license terms.
 # The reproduction in any form without written permission of
 # Doctor Web, Ltd. and proper attribution is strongly prohibited.
 #######################################################################


#vars 

if [ -f /usr/xpg4/bin/awk ]; then
    AWK=/usr/xpg4/bin/awk
else
    AWK=awk
fi

if [ -f /usr/xpg4/bin/sed ]; then
    SED=/usr/xpg4/bin/sed
else
    SED=sed
fi

if [ -f /usr/xpg4/bin/grep ]; then
    GREP=/usr/xpg4/bin/grep
else
    GREP=grep
fi

if [ -f /usr/xpg4/bin/egrep ]; then
    EGREP=/usr/xpg4/bin/egrep
else
    EGREP=egrep
fi

DEBUG="no"
#ROOT_PATH=""
DEFAULT_BIN_PATH=/opt/drweb
DEFAULT_BIN_PATH=`echo "$DEFAULT_BIN_PATH" | $SED 's/^\///'`
SCRIPTS_DIR="$DEFAULT_BIN_PATH/maild/scripts"
DOC_DIR="$DEFAULT_BIN_PATH/doc/maild"
DEFAULT_VAR_PATH="$ROOT_PATH/var/drweb"
if [ "0" = "1" ]; then
   INIT_SCRIPT="$ROOT_PATH/usr/local/etc/rc.d/00.drweb-monitor.sh"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/usr/local/drweb/lib"
else
   INIT_SCRIPT="$ROOT_PATH/etc/init.d/drweb-monitor"
   SYS_LIB_DIR="/usr/local/lib"
   DW_LIB_DIR="$ROOT_PATH/opt/drweb/lib"
fi

#functions
######################################################################
dwd()
{
    if [ "$DEBUG" = "yes" ]; then
        echo $1
    fi
}
######################################################################
DwFindFullPath()
{
    if [ -x "./$0" ] ; then
        var=`echo $0`
        var=`echo "$var" | $SED 's/^\.\///'`
        SCRIPT_PATH=`pwd`/"$var"
        SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\"//g'`
    else
        SCRIPT_PATH="$0"
    fi
    SCRIPT_NAME=`echo "$SCRIPT_PATH" | $SED 's/^.*\/\([^\/]*\)$/\1/'`
    SCRIPT_PATH=`echo "$SCRIPT_PATH" | $SED 's/\/[^\/]*$//'`
    dwd "SCRIPT_PATH=$SCRIPT_PATH SCRIPT_NAME=$SCRIPT_NAME"
}

######################################################################
DwInstall()
{
(cd "$1"
    CHOWN=""
    ADD="no"
    for element in *; do
        if [ -d "$element" ]; then
            if [ ! -d "$2/$element" ]
            then
                if [ "$3" = "modif" -o "$3" = "update" ]
                then
                    mkdir -p "$2/$element" || exit 1
                    chown drweb:drweb "$2/$element" || exit 1
                    chmod 0775 "$2/$element" || exit 1
                    ADD="yes"
                    dwd "$element => $2/$element"
                fi
            fi
            DwInstall $element "$2/$element" $3 || exit 1
            if [ "$ADD" = "yes" ]
            then
                echo "$2/$element" >> "$DW_INSTALL_LOG"
            fi
        fi
        suffix=""
        if [ -f "$element"  -a "$element" != "$SCRIPT_NAME"  -a "$element" != "update.sh" ]; then
            ext=`echo "$element" | $SED 's/^.*\.\([^\.]*\)$/\1/'`
            if [ -f "$2/$element" ]; then
                if [ "$element" = "emails" -o "$ext" = "conf" -o "$ext" = "ini" -o "$ext" = "mmc" ]; then
                    if [ "$3" = "modif" ]
                    then
                        echo "File $2/$element exist: save it to $2/$element.drwebsave"
                        mv "$2/$element" "$2/$element.drwebsave" || exit 1
                    elif [ "$3" = "update" ]
                    then
                        echo "File $2/$element exist: save new version to $2/$element.new"
                        suffix=".new"
                    fi
                elif [ "$3" != "update" ]; then
                    echo "File $2/$element already exist: can not continue!"
                    exit 1
                fi
            fi
            if [ "$3" != "fake" ]
            then
                if [ ! -f "$2/$element$suffix" ]; then
                    echo "$2/$element$suffix" >> "$DW_INSTALL_LOG"
                    dwd "$element ($ext) => $2/$element$suffix"
                fi
                if [ -L "$element" ]; then
                    STAT_INFO=`ls -l "$element" | $SED "s/^.*-> \(.*\)$/\1/"`
                    (cd "$2"; if [ -f "$element$suffix" ]; then rm -f "$element$suffix"; fi; ln -s "$STAT_INFO" "$element$suffix"|| exit 1) || exit 1
                else
                    cp -p -R "$element" "$2/$element$suffix" || exit 1
                fi
                CHOWN="$CHOWN $2/$element$suffix"
            fi
        fi
    done
    if [ ! -z "$CHOWN" ]; then
        chown drweb:drweb $CHOWN || exit 1
    fi
)
}
######################################################################
DwStop()
{
    if [ ! -f $INIT_SCRIPT ]; then
        echo "Can not find init script $INIT_SCRIPT"
        exit 2
    fi
    
    $INIT_SCRIPT status
    if [ $? -eq 0 ]; then
        $INIT_SCRIPT stop
        if [ $? -ne 0 ]; then
            echo "Can not stop Dr.Web MailD"
            exit 3
        fi
    fi
}
######################################################################
DwReload()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT reload
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers reloaded"
            else
                echo "" 
                echo "Automatic reload failed."
                echo "Please inspect Dr.Web for Unix mail servers and plugins configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    fi
}
######################################################################
DwRestart()
{
    if [ -x $INIT_SCRIPT ]; then
        $INIT_SCRIPT status
        if [ $? -eq 0 ]; then
            $INIT_SCRIPT restart
            if [ $? -eq 0 ]; then
                echo "===> Dr.Web for Unix mail servers restarted"
            else
                echo "" 
                echo "Automatic start failed."
                echo "Please inspect Dr.Web for Unix mail servers configuration for default values."
                echo "Please start drweb-monitor manually."
            fi
        fi
    elif [ -x "`which svcadm`" ] && svcs drweb-monitor >/dev/null 2>&1; then
        # Would not restart if disabled currently.
        svcadm restart drweb-monitor
    else
        echo ""
        echo "Dr.Web for Unix mail servers restart failed."
        echo "Neither SysV nor SVC service was found on your system."
    fi
}

######################################################################
DwBackup()
{
    if [ -f "$1" ] ; then

        if [ -f "$1.drwebsave" ] ; then

            CONF_DRWEBSAVE="yes"
            TIMESTAMP=`date "+%Y%m%d-%k_%M_%S"`
            echo ""
            echo "Info: $1.drwebsave exists. Saving $1 as $1.$TIMESTAMP"
            echo ""
            cp "$1" "$1.$TIMESTAMP"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 3
            fi
        fi
        if [ "$CONF_DRWEBSAVE" != "yes" ] ; then
            cp "$1" "$1.drwebsave"
            if [ $? -ne 0 ] ; then
                echo ""
                echo "Error: can't backup $1! Installation aborted."
                echo ""
                return 2
            else
                echo ""
                echo "Success: backup of $1 in $1.drwebsave."
                echo ""
            fi

        fi
    else
        echo ""
        echo "Error: can't find $1!"
        echo ""
        return 1
    fi
    return 0;
}
######################################################################
DwPostProc()
{
    chmod +s "$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-inject"
}

######################################################################
DwInstallLibs()
{
:
#     for file in libstdc++.so.6 libgcc_s.so.1
#     do
#       if [ ! -f $SYS_LIB_DIR/$file ]; then
#           dwd "Make link from $DW_LIB_DIR/$file to $SYS_LIB_DIR/$file"
#           rm -f $SYS_LIB_DIR/$file >/dev/null 2>&1
#           ln -s $DW_LIB_DIR/$file $SYS_LIB_DIR/$file || exit 1
#       fi.
#     done
}
######################################################################
DwSetPluginName()
{
    OLD=`pwd`
    if [ ! -d "/$PLUGINS_DIR" ];then
        echo "Can not find plugin directory '/$PLUGINS_DIR'"
        exit 1
    fi
    cd "/$PLUGINS_DIR"
    for plugin in *; do
        PLUGIN_NAME=`echo "$plugin" | $SED 's/^lib\(.*\)\.so$/\1/'`
        dwd "PLUGIN_NAME = $PLUGIN_NAME"
        break
    done
    cd "$OLD"
    if [ "x$PLUGIN_NAME" = "x*" ]; then
        echo "Can not find any plugin library in /$PLUGINS_DIR"
        exit 1
    fi
}

DwConfigureMonitor()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/User = drweb/User = root/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^User = root$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                if test -z $POSTINSTALL_FLAG; then
                    DwRestart
                fi
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change User = drweb to User = root manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

DwConfigureMonitorRunAppList()
{
    DEFAULT_CONF_PATH=/etc/drweb
    if test -f "$DEFAULT_CONF_PATH/monitor.conf" ; then
        RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
        if test -z "$RES" ; then
	    MONITOR_CONF_SAVE="$DEFAULT_CONF_PATH/monitor.conf".drwebsave-`date +%Y%m%d_%H-%M-%S`
	    cp "$DEFAULT_CONF_PATH/monitor.conf" "$MONITOR_CONF_SAVE"
            $SED "s/RunAppList = MAILD/RunAppList = MAILDESK/" "$MONITOR_CONF_SAVE" > "$DEFAULT_CONF_PATH/monitor.conf"
            RES=`cat $DEFAULT_CONF_PATH/monitor.conf | $EGREP "^RunAppList = MAILDESK$"| head -n1`
            if test -n "$RES" ; then
                echo "$DEFAULT_CONF_PATH/monitor.conf patched."
                DwRestart
            else
                echo "$DEFAULT_CONF_PATH/monitor.conf not patched. Change 'RunAppList = MAILD' to 'RunAppList = MAILDESK' manually if needed."
            fi
        else
            echo "$DEFAULT_CONF_PATH/monitor.conf already patched."
        fi
    fi
}

############ PRE ##############

#root

id | $EGREP "uid=0\(" 1> /dev/null
if [ $? -ne 0 ]; then
    echo "You must be root to run the Dr.Web for Unix mail servers installer."
    exit 1
fi

#reset umask
umask 0002

cd "/$SCRIPTS_DIR"

DwFindFullPath


#vars
MTA_CONFIGURE="no"
DW_INSTALL_LOG="$DEFAULT_VAR_PATH/maild_COURIER_install.info"

DwPrepareMta()
{
:
}
DwUpdateMTA()
{
    DEAFULT_USERNAME="daemon"
    id "courier" 2>/dev/null | $EGREP "uid=0\(" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        DEAFULT_USERNAME="courier"
    fi
    echo "Enter username used to run Courier MTA [default=$DEAFULT_USERNAME]"
    read INPUT
    if [ "x$INPUT" = "x" ]; then
        INPUT=$DEAFULT_USERNAME
    fi

    COURIER_BIN_REAL="$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-courier.real"
    COURIER_BIN="$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-courier"

    if [ -L "$COURIER_BIN" ] ; then
        chown $INPUT:drweb "$COURIER_BIN_REAL" || exit 1
        chmod 6771 "$COURIER_BIN_REAL" || exit 1
    else if [ -f "$COURIER_BIN" ] ; then
        chown $INPUT:drweb "$COURIER_BIN" || exit 1
        chmod 6771 "$COURIER_BIN" || exit 1
    fi fi
}
DwInstallMtaCOURIER()
{

    if [ -d "/usr/lib/courier" ] ; then
        COURIER_DIR="/usr/lib/courier"
    elif [ -d "/usr/local/courier" ] ; then
        COURIER_DIR="/usr/local/courier"
    elif [ -d "/usr/local/share/courier" ] ; then
        COURIER_DIR="/usr/local/share/courier"
    else
        echo ""
        echo "Error: courier directory was not found!"
        return 1
    fi
    if [ -d "/usr/lib/courier/libexec/filters" ] ; then
        FILTERS_DIR="/usr/lib/courier/libexec/filters"
    elif [ -d "/usr/local/courier/libexec/filters" ] ; then
        FILTERS_DIR="/usr/local/courier/libexec/filters"
    elif [ -d "/usr/local/share/courier/libexec/filters" ] ; then
        FILTERS_DIR="/usr/local/share/courier/libexec/filters"
    elif [ -d "/usr/local/libexec/filters" ] ; then
        FILTERS_DIR="/usr/local/libexec/filters"
    elif [ -d "/usr/lib/courier/filters" ] ; then
        FILTERS_DIR="/usr/lib/courier/filters"
    else
        echo ""
        echo "Error: directory courier filters was not found!"
        return 1
    fi

    if [ -f "/usr/lib/courier/sbin/filterctl" ] ; then
        FILTER_CTL="/usr/lib/courier/sbin/filterctl"
    elif [ -f "/usr/local/courier/sbin/filterctl"  ] ; then
        FILTER_CTL="/usr/local/courier/sbin/filterctl"
    elif [ -f "/usr/local/share/courier/sbin/filterctl"  ] ; then
        FILTER_CTL="/usr/local/share/courier/sbin/filterctl"
    elif [ -f "/usr/local/sbin/filterctl"  ] ; then
        FILTER_CTL="/usr/local/sbin/filterctl"
    elif [ -f "/usr/lib/courier/filterctl"  ] ; then
        FILTER_CTL="/usr/lib/courier/filterctl"
    else
        echo ""
        echo "Error: filterctl was not found!"
        return 1
    fi

    if [ -d "/usr/local/courier/etc" ] ; then
        ENABLEFILTERING_DIR="/usr/local/courier/etc"
    elif [ -d "/usr/lib/courier/etc" ] ; then
        ENABLEFILTERING_DIR="/usr/lib/courier/etc"
    elif [ -d "/usr/local/share/courier/etc" ] ; then
        ENABLEFILTERING_DIR="/usr/local/share/courier/etc"
    elif [ -d "/usr/local/etc/courier" ] ; then
        ENABLEFILTERING_DIR="/usr/local/etc/courier"
    elif [ -d "/etc/courier" ] ; then
        ENABLEFILTERING_DIR="/etc/courier"
    else
        echo ""
        echo "Error: directory, where enablefiltering file must be, was not found!"
        return 1
    fi

    if [ -f "/usr/lib/courier/sbin/courierfilter" ] ; then
        COURIER_FILTER="/usr/lib/courier/sbin/courierfilter"
    elif [ -f "/usr/local/courier/sbin/courierfilter" ] ; then
        COURIER_FILTER="/usr/local/courier/sbin/courierfilter"
    elif [ -f "/usr/local/share/courier/sbin/courierfilter" ] ; then
        COURIER_FILTER="/usr/local/share/courier/sbin/courierfilter"
    elif [ -f "/usr/local/sbin/courierfilter" ] ; then
        COURIER_FILTER="/usr/local/sbin/courierfilter"
    elif [ -f "/usr/sbin/courierfilter" ] ; then
        COURIER_FILTER="/usr/sbin/courierfilter"
    else
        echo ""
        echo "Error: courierfilter was not found!"
        return 1
    fi

    $COURIER_FILTER stop
    unlink $FILTERS_DIR/drweb-courier  >/dev/null 2>&1

    DEAFULT_USERNAME="daemon"
    id "courier" 2>/dev/null | $EGREP "uid=0\(" >/dev/null 2>&1
    if [ $? -eq 0 ]; then
        DEAFULT_USERNAME="courier"
    fi
    echo "Enter username used to run Courier MTA [default=$DEAFULT_USERNAME]"
    read INPUT
    if [ "x$INPUT" = "x" ]; then
        INPUT=$DEAFULT_USERNAME
    fi

    COURIER_BIN_REAL="$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-courier.real"
    COURIER_BIN="$ROOT_PATH/$DEFAULT_BIN_PATH/drweb-courier"

    if [ -L "$COURIER_BIN" ] ; then
        chown $INPUT:drweb "$COURIER_BIN_REAL" || exit 1
        chmod 6771 "$COURIER_BIN_REAL" || exit 1
    else if [ -f "$COURIER_BIN" ] ; then
        chown $INPUT:drweb "$COURIER_BIN" || exit 1
        chmod 6771 "$COURIER_BIN" || exit 1
    fi fi

#fix perm on /var/drweb/* for courier (#0029737)
    if ( which find > /dev/null 2>&1 ) ; then
	which xargs >/dev/null 2>&1 && find /var/drweb/ -type d | xargs chmod 2775
    fi
    ln -s $ROOT_PATH/$DEFAULT_BIN_PATH/drweb-courier $FILTERS_DIR

    if [ -f "$FILTERS_DIR/drweb-courier" ] ; then
        $FILTER_CTL stop drweb-courier
        $FILTER_CTL start drweb-courier
    else
        echo ""
        echo "Error: $FILTERS_DIR/drweb-courier file was not found!"
        return 1
    fi

    ENABLEFILTERING_FILE="$ENABLEFILTERING_DIR/enablefiltering"
    if [ -f "$ENABLEFILTERING_FILE" ] ; then
        
        DwBackup "$ENABLEFILTERING_FILE" || return 1

        IS_ESMTP=`cat $ENABLEFILTERING_FILE | grep esmtp`
        IS_LOCAL=`cat $ENABLEFILTERING_FILE | grep local`
        if [ "x$IS_ESMTP" = "x" ] ; then
            CONTENT_ENABLEFILTERING=`cat $ENABLEFILTERING_FILE`
            echo "$CONTENT_ENABLEFILTERING esmtp" > $ENABLEFILTERING_FILE
            echo ""
            echo "Info: $ENABLEFILTERING_FILE patched for esmtp filtering...       OK"
        else
            echo ""
            echo "Info: esmtp filter already enabled in $ENABLEFILTERING_FILE."
        fi
        if [ "y$IS_LOCAL" != "y" ] ; then
            CONTENT_ENABLEFILTERING=`cat $ENABLEFILTERING_FILE`
            echo "$CONTENT_ENABLEFILTERING" | $SED 's/local//' > $ENABLEFILTERING_FILE
            echo ""
            echo "Info: $ENABLEFILTERING_FILE patched for remove local filtering...       OK"
        fi
    else
        ENABLEFILTERING_FILE="$ENABLEFILTERING_DIR/enablefiltering"
        echo "esmtp" >> $ENABLEFILTERING_FILE
        echo ""
        echo "Info: $ENABLEFILTERING_FILE created...       OK"
    fi

    (
        DW_ROOT_PATH=$ROOT_PATH;
        export DW_ROOT_PATH;
        cd "$ROOT_PATH/$SCRIPTS_DIR"
        echo "$COURIER_DIR" | perl -e '$COURIER_DIR=<>;chomp $COURIER_DIR;use dwconfig;my ($dw_root_path,$dw_bin_dir,$dw_etc_dir,$dw_conf_file) = DwInit();my $drweb_conf = DwLoadFile("$dw_etc_dir/$dw_conf_file");if(-d "/usr/local/share/courier"){$BaseDir="/usr/local/share/courier";}else{$BaseDir=$COURIER_DIR;};$drweb_conf = DwModify($drweb_conf,"Courier","BaseDir",$BaseDir,"string");if( -d "/var/spool/courier/allfilters"){$SocketDirs="/var/spool/courier/allfilters, /var/spool/courier/filters";}elsif( -d "/var/lib/courier/allfilters"){$SocketDirs="/var/lib/courier/allfilters, /var/lib/courier/filters";}else{$SocketDirs="$COURIER_DIR/var/allfilters, $COURIER_DIR/var/filters"};$drweb_conf = DwModify($drweb_conf,"Courier","SocketDirs",$SocketDirs,"vector");DwSaveFile("$dw_etc_dir/$dw_conf_file",$drweb_conf);'
        )
    echo "drweb-courier successfull installed"
    MTA_CONFIGURE="yes"
}



############ START ##############

if [ "x$1" = "xupdate" ] ; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will update configuration of COURIER.
    echo
    while true ; do
        echo "Do you wish to continue?"
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DWPrepareMta
    DwUpdateMTA
fi


DwPrepareMta

#configure MTA
if  [ "x$SKIP_MTA_CONFIG" != "xyes" ]; then
    echo
    echo ------------------------------------------------------------
    echo     Welcome to the Dr.Web InstallShield Wizard.
    echo
    echo The InstallShield Wizard will configure COURIER.
    echo
    while true ; do
        echo "Perform MTA configuration?"
	echo "Please enter yes or no."
        read yesno
        case "$yesno" in
            y | yes | Y | Yes | YES)
            break
            ;;
            n | no | N | No | NO)
            echo "Try later. Bye!"
            exit 1
            ;;
            *)
            echo Please enter yes or no.
            ;;
        esac
    done
    DwInstallMtaCOURIER
fi

if [ "COURIER" != "MAILDESK" ]; then
  if [ "$MTA_CONFIGURE" = "no" ]
  then
    echo "Please, refer to documentation on COURIER adjustment residing in $ROOT_PATH/$DOC_DIR directory."
  else
    echo "Please, restart COURIER manually."
  fi
fi

